# Slune
# Copyright (C) 2002-2003 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from __future__ import generators

import time, random, math

import soya, _soya, soya.soya3d as soya3d, soya.model as model, soya.particle as particle, soya.game.level as game_level, soya.widget as widget, soya.ray as ray
import py2play.level as py2p_level, py2play.action as action, py2play.character as character
from soya.math3d import Point, Vector
import slune.character as slune_character
import slune.controler
import slune.globdef as globdef, slune.sound as sound, slune.level as slune_level

class Medoc(soya3d.Volume):
  radius = 2.5
  
  def __init__(self, parent = None):
    soya3d.Volume.__init__(self, parent, model.Shape.get("medoc2"))
    
  def begin_round(self):
    for character in self.parent.characters:
      if isinstance(character, slune_character.Competitor) and (character.distance_to(self) < self.radius):
        sound.play("flag-1.wav", character)
        firework = particle.FlagSubFire(character.level, nb_particles = 8)
        firework.move(self)
        firework.regenerate()
        firework.removable = 1
        
        if hasattr(character, "nb_medoc"):
          character.nb_medoc += 1
          if character.nb_medoc >= self.nb_max:
            from py2play.idler import IDLER
            IDLER.level_completed(character, 1, _("__medoc_winner__"))
            
        else:
          character.nb_medoc = 1
          #if character.vehicle == 2:
          #  character.medoc = soya3d.Volume(character.internal, model.Shape.get("medoc2"))
            
        self.parent.remove(self)
        

def add_medocs(level, nb, minimap = 1):
  down = Vector(level, 0.0, -1.0, 0.0)
  
  for i in range(nb):
    m = Medoc(level)
    m.nb_max = nb
    while 1:
      m.set_xyz(level.random.uniform(5.0, 200.0), 1000.0, level.random.uniform(5.0, 200.0))
      
      r = level.raypick(m, down, -1, 3)
      if r:
        r[0].convert_to(level)
        m.y = r[0].y + 0.5
        break
      
  from py2play.idler import IDLER
  if minimap: IDLER.no_blackbands_group.insert(0, MiniMap(level))
  

class MiniMap(widget.Widget):
  def __init__(self, level):
    self.level = level
    
    self.top = 10
    self.width  = 200
    self.height = 200
    self.left = soya.root_widget.width - self.width - 40
    
    medocs = filter(lambda x: isinstance(x, Medoc), level)
    min_x = min_y =  10e1000
    max_x = max_y = -10e1000
    for medoc in medocs:
      if medoc.x < min_x: min_x = medoc.x
      if medoc.x > max_x: max_x = medoc.x
      if medoc.z < min_y: min_y = medoc.z
      if medoc.z > max_y: max_y = medoc.z
    f_x = self.width  / (max_x - min_x)
    f_y = self.height / (max_y - min_y)
    if f_x < f_y:
      f = f_x
      left = self.left
    else:
      f = f_y
      left = self.left + ((max_y - min_y) - (max_x - min_x)) * f
    top = self.top
    
    self.f     = f
    self.cleft = left
    self.ctop  = top
    self.min_x = min_x
    self.min_y = min_y
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.left = parent_width - self.width - 40
    self.screen_width  = parent_width
    self.screen_height = parent_height
    
  def render(self):
    _soya.glColor4f(1.0, 1.0, 1.0, 1.0)
    _soya.glBegin(_soya.GL_QUADS)
    for medoc in filter(lambda x: isinstance(x, Medoc), self.level):
      x = self.cleft + self.f * (medoc.x - self.min_x)
      y = self.ctop  + self.f * (medoc.z - self.min_y)
      _soya.glVertex2f(x - 4.0, y - 4.0)
      _soya.glVertex2f(x - 4.0, y + 4.0)
      _soya.glVertex2f(x + 4.0, y + 4.0)
      _soya.glVertex2f(x + 4.0, y - 4.0)
      
    _soya.glEnd()
    
    for character in self.level.characters:
      if isinstance(character, slune_character.Competitor):
        x = self.cleft + self.f * (character.x - self.min_x)
        y = self.ctop  + self.f * (character.z - self.min_y)
        
        material = model.Material.get("head_" + character.perso_name + "_1")
        material.activate()
        _soya.glEnable(_soya.GL_BLEND)
        _soya.glBegin(_soya.GL_QUADS)
        _soya.glTexCoord2f(0.0, 0.0); _soya.glVertex2f(x - 20.0, y - 20.0)
        _soya.glTexCoord2f(0.0, 1.0); _soya.glVertex2f(x - 20.0, y + 20.0)
        _soya.glTexCoord2f(1.0, 1.0); _soya.glVertex2f(x + 20.0, y + 20.0)
        _soya.glTexCoord2f(1.0, 0.0); _soya.glVertex2f(x + 20.0, y - 20.0)
        _soya.glEnd()
        _soya.glDisable(_soya.GL_BLEND)
        _soya.activate_material(None)
